
/******************************************************************************
* MODULE     : mml_gmp.hpp
* DESCRIPTION: Interface to gmp and mpfr
* COPYRIGHT  : (C) 2003  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#ifndef __MML_GMP_HPP
#define __MML_GMP_HPP
#include <gmp.h>
#include <mpfr.h>
#include <math.h>
#include "base.hpp"

namespace mml {

#define BITS_PER_LIMB  __GMP_BITS_PER_MP_LIMB
#define MIN_PREC       MPFR_PREC_MIN
#define MIN_SECURITY   2
#define DEFAULT_PREC   mml_default_prec
#define DEFAULT_ROUND  mml_default_round
#define DEFAULT_LOW    mml_default_low

extern mp_prec_t mml_default_prec;
extern int       mml_default_round;
extern mp_prec_t mml_default_low;

extern ostream& print (ostream& out, const mpz_t x);
extern ostream& print (ostream& out, const mpfr_t x, mp_prec_t bits= 0);
extern ostream& operator << (ostream& out, const mpfr_t x);

inline void mpz_mul_2si (mpz_t dest, mpz_t arg, long int shift) {
  if (shift >= 0) mpz_mul_2exp (dest, arg, shift);
  else mpz_fdiv_q_2exp (dest, arg, -shift);
}
inline void mpq_mul_2si (mpq_t dest, mpq_t arg, long int shift) {
  if (shift >= 0) mpq_mul_2exp (dest, arg, shift);
  else mpq_div_2exp (dest, arg, -shift);
}

inline void mpfr_set_ui_2si (mpfr_t dest, mp_limb_t k, mp_exp_t p, int mode) {
  mpfr_set_ui (dest, k, mode);
  mpfr_mul_2si (dest, dest, p, mode); }
inline void mpfr_mid (mpfr_t dest, mpfr_t x1, mpfr_t x2, int mode) {
  mpfr_add (dest, x1, x2, mode);
  mpfr_mul_2si (dest, dest, -1, mode); }
inline void mpfr_rad (mpfr_t dest, mpfr_t x1, mpfr_t x2, int mode) {
  mpfr_sub (dest, x2, x1, mode);
  mpfr_mul_2si (dest, dest, -1, mode); }
inline mp_exp_t mpfr_get_step (mpfr_t arg) {
  return mpfr_get_exp (arg) - mpfr_get_prec (arg); }
inline mp_prec_t mpfr_prec_round_up (mp_prec_t prec) {
  return BITS_PER_LIMB * ((prec + BITS_PER_LIMB - 1) / BITS_PER_LIMB); }

inline signed int min (signed int i, signed int j) {
  return i<j? i: j; }
inline signed int max (signed int i, signed int j) {
  return i>j? i: j; }
inline unsigned int min (unsigned int i, unsigned int j) {
  return i<j? i: j; }
inline unsigned int max (unsigned int i, unsigned int j) {
  return i>j? i: j; }
inline signed short int min (signed short int i, signed short int j) {
  return i<j? i: j; }
inline signed short int max (signed short int i, signed short int j) {
  return i>j? i: j; }
inline unsigned short int min (unsigned short int i, unsigned short int j) {
  return i<j? i: j; }
inline unsigned short int max (unsigned short int i, unsigned short int j) {
  return i>j? i: j; }
inline signed long int min (signed long int i, signed long int j) {
  return i<j? i: j; }
inline signed long int max (signed long int i, signed long int j) {
  return i>j? i: j; }
inline unsigned long int min (unsigned long int i, unsigned long int j) {
  return i<j? i: j; }
inline unsigned long int max (unsigned long int i, unsigned long int j) {
  return i>j? i: j; }

}
#endif // __MML_GMP_HPP
